/****************************************************************************
* NVMC                                                                      *
* EnVy My Car																*
*                                                                           *
* Fabio Ganovelli                                                           *
*                                                                           *
* All rights reserved.                                                      *
*                                                                           *
* This program is free software; you can redistribute it and/or modify      *
* it under the terms of the GNU General Public License as published by      *
* the Free Software Foundation; either version 2 of the License, or         *
* (at your option) any later version.                                       *
*                                                                           *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
* GNU General Public License (http://www.gnu.org/licenses/gpl.txt)          *
* for more details.                                                         *
*                                                                           *
****************************************************************************/




/* some standard concepts*/
#include <math.h>
#include <time.h>
#include <vector>

/* inclusion from the VCG of the concept of point3  */
#include <vcg/space/point3.h>

/* library for graphics*/
#include <GL/glew.h>	// handles the OpenGL extensions
#include <GL/glut.h>	// interfaces toward the operating system
#include <GL/glu.h>		// utilities (wrapping of OpenGL) 

/* inclusion of the nvmc files */
#include "client_game.h"	// all the concepts we need
	

/** Client  */
extern  NvmcClient cli;

/* value of pi */
#define M_PI       3.14159265358979323846

/** viewport size */
int width,height;

/* send a snapshot */
bool sendSS = false;

/** inizializzazione lo stato di opengl */
void init_opengl(){
	glPolygonMode(GL_FRONT,GL_FILL); // rasterizza solo i contorni dei poligoni
}

/** callback function for viewport resize.*/
void resize_screen(int w, int h){

	width = w;
	height = h;
	glViewport(0,0,width,height);
}


/* draw reference system*/
void draw_axis(){

	glPushAttrib(0xffffffff);
	glDisable(GL_LIGHTING);
	glBegin(GL_LINES);
	glColor3f(1.0,0.0,0.0);
  glVertex3f(0,0,0); //   X
  glVertex3f(10,0,0); 
  
	glColor3f(0.0,1.0,0.0);
  glVertex3f(0,0,0); //   Y
  glVertex3f(0,10,0);
  
	glColor3f(0.0,0.0,1.0);
  glVertex3f(0,0,0); //   Z
  glVertex3f(0,0,10);
  glEnd();
 	glPopAttrib();
}

/* print a string on the screen*/
void gl_print(float x, float y, const char *str) {
  glRasterPos2f(x, y);
  int len = (int)strlen(str);
  for(int i = 0; i < len; i++) 
    glutBitmapCharacter(GLUT_BITMAP_HELVETICA_18, str[i]);
}

/** render the scene */
void draw_scene(){
	static int cn = 0;
	static int part = 0;
	glClear(GL_COLOR_BUFFER_BIT);

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	/* here take the bounding box of the whole circuit */
	vcg::Point3f bmin = race.circuit->bbox.min;
	vcg::Point3f bmax = race.circuit->bbox.max;

	float sidex,sidez;
	sidex = bmax[0]-bmin[0];
	sidez = bmax[2]-bmin[2];

	// this function asks nvmc to update the state of the player with the information 
	// received from the server
	cli.UpdatePlayers();

	// set the camera

	Race::PlayersIterator  hh  = cli.local_player;

	if( hh != cli.players.end()){
		glOrtho(-6,6,-6,6,0.0,10.0);
		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();

		vcg::Point4f pos = (*hh).car.frame.O();
		gluLookAt(pos[0],6.0,pos[2],pos[0],pos[1],pos[2],0,0,-1);
	}
	else// if there are no player just render the circuit from above
	{
		glOrtho(-sidex*0.5,sidex*0.5,-sidez*0.5,sidez*0.5,0.0,400);
		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();
		gluLookAt(0,400,0,0,0,0,0,0,-1);
	}

	// draw bounding box of the terrain
	glColor3f(0.1,0.2,0.4);
	glBegin(GL_QUADS);
	glVertex3f(bmin[0],bmin[1],bmin[2]);
	glVertex3f(bmax[0],bmin[1],bmin[2]);
	glVertex3f(bmax[0],bmin[1],bmax[2]);
	glVertex3f(bmin[0],bmin[1],bmax[2]);
	glEnd();

	// draw the axis
	glPushMatrix();
	glScalef(bmax[0]-bmin[0],bmax[1]-bmin[1],bmax[2]-bmin[2]);
	draw_axis();
	glPopMatrix();

	// draw the circuit	
	Street::TilesIterator ti;
	int i;
	glColor3f(1.0,0.0,0.0);
	glPolygonMode(GL_FRONT,GL_LINE);
	glBegin(GL_QUAD_STRIP);
	for(i=0,ti = race.circuit->street.path.begin(); ti != race.circuit->street.path.end();++ti,++i){
		vcg::Point3f p0,p1;
		p0 = (*ti)[0].P(); p1 = (*ti)[1].P();
		glVertex3f(p0[0],p0[1]+0.1,p0[2]);
		glVertex3f(p1[0],p1[1]+0.1,p1[2]);
	}
	glEnd();

	// draw the tunnels	
	Street::TunnelsIterator  tni;
	glColor3f(1.0,1.0,0.0);
	glPolygonMode(GL_FRONT,GL_FILL);
	for(i=0,tni = race.circuit->street.tunnels.begin(); tni != race.circuit->street.tunnels.end();++tni,++i){
		glBegin(GL_QUAD_STRIP);
		for(i=0,ti = (*tni).path.begin(); ti != (*tni).path.end();++ti,++i)
			{
				vcg::Point3f p0,p1;
				p0 = (*ti)[0].P(); p1 = (*ti)[1].P();
				glVertex3f(p0[0],p0[1]+0.2,p0[2]);
				glVertex3f(p1[0],p1[1]+0.2,p1[2]);
			}
	glEnd();
	}

	// draw the buildings
	Circuit::BuildingsIterator bi;
	glColor3f(1.0,1.0,1.0);
	glPolygonMode(GL_FRONT,GL_FILL);
	for(bi = race.circuit->buildings.begin(); bi != race.circuit->buildings.end(); ++bi){
		glBegin(GL_POLYGON);
		for(unsigned i=0; i < (*bi).footprint.size();++i)
			glVertex3f((*bi).footprint[i].P()[0],(*bi).footprint[i].P()[1],(*bi).footprint[i].P()[2]);
		glEnd();
	}

	// draw the trees
	Circuit::TreesIterator tsi;
	glColor3f(10.0,1.0,0.0);
	glBegin(GL_POINTS);
	for(tsi = race.circuit->trees.begin(); tsi != race.circuit->trees.end(); ++tsi)
		glVertex3f((*tsi).pos[0],(*tsi).pos[1],(*tsi).pos[2]);
	glEnd();

	// draw the players
	Race::PlayersIterator pi;

	for(pi = cli.players.begin(); pi != cli.players.end(); ++pi)
		{
			glPushMatrix();
			/* apply transformation coded in the frame */
			Frame f = ((*pi).car.frame);
			glMultMatrixf(&f[0][0]);
			glPushMatrix();
			glColor3f(1.0,1.0,1.0);

			glColor3f(0,1,0);
			glBegin(GL_LINES);
			glVertex3f(0.0,1.0,1.0);
			glVertex3f(0.0,1.0,-1.0);
			glEnd();

			glColor3f(0,0,1);
			glTranslatef(0.0,0.0,(*pi).car.front_wheel_pos);
			glRotatef((*pi).car.steering_angle*180.0/ M_PI ,0,1,0);
			glBegin(GL_LINES);
			glVertex3f(0.0,1.0,0.4);
			glVertex3f(0.0,1.0,-0.4);
			glEnd();

			glPopMatrix();
			glColor3f(1.0,0.0,0.0);
			gl_print(0.0,0.0,(char*)(*pi).ip_address.toString().toAscii().data()); // write the IP address of the client
			glPopMatrix();
			

	 }

	glLoadIdentity();
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	/* print the commands */
	glColor3f(1.0,1.0,0.0);
	gl_print(-1,0.8,"accel: w");
	gl_print(-1,0.7,"slow:  s");
	gl_print(-1,0.6,"left:  a");
	gl_print(-1,0.5,"right: d");
	gl_print(-1,0.4,"brake: x");
	gl_print(-1,0.3,"reset: p");

	if(sendSS){
		cli.SendScreenshot();
		sendSS=false;
	}
	cli.DrawScreenshots(std::string());
	glutSwapBuffers();
	cn++;
	if(clock()-part >CLOCKS_PER_SEC){	
		printf("VIEW fps %d\n",cn);
		part = clock();
		cn=0;
	}
}



///
//bool crtlpressed=false;
void keys(unsigned char e, int i, int j){
	switch(e ){
		case 'w': cli.Command(COMMAND_INCREASE_SPEED);break;
		case 's': cli.Command(COMMAND_DECREASE_SPEED);break;
		case 'a': cli.Command(COMMAND_TURN_LEFT);break;
		case 'd': cli.Command(COMMAND_TURN_RIGHT);break;
		case 'x': cli.Command(COMMAND_BRAKE);break;
		case 'p': cli.Command(COMMAND_RESTART);break;
		case 'g': sendSS = true; break;
	}
}
void run_forever(){
	static int last_rendering = 0;
	draw_scene();	
	glutPostRedisplay();
  
}
///
int launch_viewer(){

	char * fake[255];
	fake[0] = new char[10];
	int pt=1;
	glutInit(&pt,&fake[0]);
	glutInitWindowSize(500,500);
	glutInitDisplayMode ( GLUT_RGBA | GLUT_DEPTH | GLUT_DOUBLE);
	glutCreateWindow("glut NVMC");

	glutKeyboardFunc(keys);
	glutReshapeFunc(resize_screen);
	glutDisplayFunc(run_forever);

	// inizializzamo lo stato di opengl
	init_opengl();
  
	glutMainLoop();

	return 0;
}

