/****************************************************************************
* NVMC                                                                      *
* EnVy My Car																*
*                                                                           *
* Fabio Ganovelli                                                           *
*                                                                           *
* All rights reserved.                                                      *
*                                                                           *
* This program is free software; you can redistribute it and/or modify      *
* it under the terms of the GNU General Public License as published by      *
* the Free Software Foundation; either version 2 of the License, or         *
* (at your option) any later version.                                       *
*                                                                           *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
* GNU General Public License (http://www.gnu.org/licenses/gpl.txt)          *
* for more details.                                                         *
*                                                                           *
****************************************************************************/


#include <qthread.h>
#include <QUdpSocket>
#include <QTcpSocket>
#include <QTcpServer>

#include <QSemaphore>

#include	<string>
#include	<time.h>
#include "process_message.h"
#include "race_simulation.h"
#include "server_game.h"
#include "simple_circuit_creator.h"
#include "messages.h"
#include "packaging.h"
#include "actions.h"
#include <map>
#include <algorithm>


NvmcServer srv;
Race race;

const int in_commands_port = 8084; // udp port to receive commands
const int out_state_port = 8082;	 // udp port to send the state
const int maxtoken = 262144; // 256k
const int in_port = 8086; // tcp port to receive snapshots
const int out_port = 8088;// tcp port to send the snapshots

extern int clockms(); 

int NvmcServer::Initialize(char * circuitFile){
	
	/* add a simple circuit to the list of circuits */
	circuits.push_back(Circuit());			
	
	if(circuitFile==NULL)  CreateCircularCircuit(circuits[0]); else circuits[0].Load(circuitFile);
	/* set the circuit as the circuit of the race */
	race.circuit = &circuits[0];
	/* set the sun position */
	circuits[0].sun_position = vcg::Point4f(10000,10000,10000,1.0);
	return 0;
};


class CommandsReceiverThread: public QThread{
public:
	CommandsReceiverThread():QThread(0){}

 void run()
 {

	QUdpSocket qs;
	qs.bind(in_commands_port);

	bool quit = false;
    do
    {	
			char data[maxtoken];
			while (!qs.waitForReadyRead());

			QHostAddress addr;
			qint64 nr  = qs.readDatagram(data,maxtoken,&addr);		

			std::string h(addr.toString().toAscii().data());
			data[nr]='\0';
			std::string msg(data);
			ProcessMessageServer(msg,h);
      // ProcessMessageServer(std::string("1"),std::string("146.48.84.176"));
       
    } while (!quit);

	
}
};

QHostAddress clients_address;
class StateSenderThread: public QThread{
public:
	StateSenderThread():QThread(0){trig = new QSemaphore(0);}

	QSemaphore * trig;

	void run()
	{
		int cl=0,part = 0;
		clients_address = QHostAddress::Broadcast;

		do{
		++cl;
		trig->acquire(); 


		if(!race.players.empty()){// rimpiazzare con un semaforo

			/* First rough draft: cycles on all the players, build a message and send it to all the players */
			std::string msg_str;
			msg_str = Pack(gTime);
			msg_str += Pack(race.circuit->sun_position);
			msg_str +=std::string(" ~ ");
			Race::PlayersIterator pi;

			race.mutex_on_players.lock();
			Race::PlayerContainer players = race.players;
			race.mutex_on_players.unlock();

			for(pi = players.begin(); pi != players.end(); ++pi){
				Player pl = (*pi);
				msg_str+=std::string(pl.ip_address.toString().toAscii().data())+std::string(" ");
				pl.mutex_on_data.lock();
				msg_str+=Pack(pl.car.frame);
				msg_str+=Pack(pl.car.vel);
				msg_str+=Pack(pl.car.steering_angle);
				msg_str+=Pack(pl.state);
				msg_str+=std::string("~");
				pl.mutex_on_data.unlock();
			}
			/* here msg_str contains the entire state of all the players */

			if(clockms()-part > 1000){
				printf("SEND fps %d\n",cl);
				part = clockms();
				cl=0;
			}
			QUdpSocket qs(0);
			
			qs.writeDatagram(msg_str.c_str(),msg_str.length(),clients_address,out_state_port);

		}
	}while(true);
	   
	
}
};

StateSenderThread s;
CommandsReceiverThread r;
void SaveFloat(char * nf,int x,int y,void * pix){
	FILE* f= fopen(nf,"wb");
	fwrite(pix,x*y*3,1,f);
	fclose(f);
}


struct Comp{
	 const bool operator ()(const Player & p1,const Player & p2)const { return p1.last_updated < p2.last_updated;}  
};

std::list<Player> allPlayers;
QSemaphore mutex_on_allPlayers;

/// thread for one-to-one tcp communication with one specific player
struct DataExchangeClientThread: public QThread{

	DataExchangeClientThread():QThread(0){}
	DataExchangeClientThread(QTcpSocket *c):QThread(false),client(c){}

	struct PlayerTime: public std::pair<Player,int>{
		PlayerTime(const Player &p,const int & t){first=p;second=t;}
	};
	// for each player the time the client "client" was updated with their snapshot
	std::map<Player,int,Comp> players_time;

	QTcpSocket * client;					// the client
	QHostAddress		client_address;		// address of the client

	void run(){

		printf("Thread started\n");
		C2SMessage msg;
		std::string data;
		int readed = 0;
		client_address = client->peerAddress();
		QString add = client_address.toString();
		std::list<Player>::iterator pi;
		while(true){

			race.mutex_on_players.lock();
			Race::PlayersIterator pi = race.FindPlayer(std::string(client_address.toString().toAscii().data()));
			bool subscribed = (pi != race.players.end());
			race.mutex_on_players.unlock();

			if(!subscribed){
				msleep(1000);
				continue;
			}

			if(client-> waitForReadyRead(1000))
			{
				/* reading of the snapshot: blocking version..it means that since
				the communcation of a snapshot starts until it is completed this thread is stuck*/	
				if( client->bytesAvailable() > sizeof(C2SMessage)){
					readed +=client->read((char*)&msg,sizeof(C2SMessage));
				
					while(client->bytesAvailable() < msg.body_length)
						client-> waitForReadyRead(10);
							
					msg.body = malloc(msg.body_length);
					readed +=	client->read((char*)msg.body,msg.body_length);
					ProcessMessageServer(msg, std::string(client_address.toString().toAscii().data()));

					printf("New screnshot from %s\n",((char*)client->peerAddress().toString().data()));
				}
			}

			race.mutex_on_players.lock();
			std::list<Player> players = race.players;
			race.mutex_on_players.unlock();

			std::map<Player,int,Comp>::iterator mpi;
			for(pi = players.begin(); pi != players.end(); ++pi)
				if( (*pi).ip_address != client_address)
			{
				mpi = players_time.find(*pi);
				if(mpi==players_time.end())
					players_time.insert(PlayerTime(*pi,-1));
				else 
					if( ((*pi).last_updated != -1) && 
						(*mpi).second  <  (*pi).last_updated){
						(*mpi).second = (*pi).last_updated;// update the time of last sent
						
						// send the screenshot to the client
						S2CMessage srvmess;
						srvmess.header = DATA_PLAYER_SNAPSHOT;
						srvmess.body_length = (*pi).ip_address.toString().length() +1+ 256*256*3;
						unsigned char l1 = (unsigned char )(*pi).ip_address.toString().length();
						client->write((char*)&srvmess,sizeof(S2CMessage));// send the message
						client->write(( char*)&l1,1);// lenght of the ipaddress string
						client->write(( char*) (*pi).ip_address.toString().toAscii().data(),(*pi).ip_address.toString().length());
						client->write(( char*)(*pi).screenshot.pixels,256*256*3);
						client->flush();
						printf("New screnshot from %s sent to %s\n",&(*pi).ip_address.toString().toAscii().data()[0],&client_address.toString().toAscii().data()[0]);

					}
				}
		}
	}
};


/// thread for one-to-one tcp communication with the players
class DataExchangeThread: public QThread{
public:
	DataExchangeThread():QThread(0){}

	void run(){
		QTcpServer datasrv;			// create the server
		datasrv.listen(QHostAddress::Any,in_port);
		while(true){

					//while( !datasrv.waitForNewConnection ( 3000 )); 
				while(!datasrv.waitForNewConnection(1000));

				QTcpSocket * client = datasrv.nextPendingConnection();


			
				DataExchangeClientThread * newone =  new DataExchangeClientThread(client);
				newone->start();

		}
	}
};

class SleepFor:public QThread { 
public:
SleepFor(int msec):QThread(0){msleep(msec);}
void run(){}
};

int run_race(void * ){
	gTime = clockms();
	int part=gTime,cl=0;
	Actions copyof_actionsToDo;
	ActionsIterator ai;
		/* first draft */
	while(true)
	{
		++cl;
	
		SleepFor(20);

		mutex_on_actionsToDo.lock();
		copyof_actionsToDo = actionsToDo;
		actionsToDo.clear();
		mutex_on_actionsToDo.unlock();

		for(ai = copyof_actionsToDo.begin(); ai != copyof_actionsToDo.end(); ++ai)
			ProcessAction(*ai);

		Race::PlayersIterator pi;

		race.mutex_on_players.lock();
		Race::PlayerContainer players = race.players;
		race.mutex_on_players.unlock();
		
		for(pi = players.begin(); pi != players.end(); ++pi) 
			Update((clockms()-gTime),*pi);
		 
		race.mutex_on_players.lock();
		race.players = players;
		race.mutex_on_players.unlock();


		if(clockms()-part>1000){
			printf("SIMU fps %d \n",cl, cl);
			cl=0;
			part = clockms();
		}

		copyof_actionsToDo.clear();
		gTime = clockms();
		s.trig->release(1);
	}
}

int NvmcServer::Start(){

	/* start the thread handling command reception (UDP)*/
	r.start();

	/* start the thread handling data reception (TCP)*/
	DataExchangeThread dt;
	dt.start();

	/* start the thread handling ipmessages sending */
	s.start();



	/* game main loop */ 
	run_race((void*)0); 
	
	return 0;
};

int NvmcServer::Terminate(){
	return 0;};
