/****************************************************************************
 * VCGLib                                                            o o     *
 * Visual and Computer Graphics Library                            o     o   *
 *                                                                _   O  _   *
 * Copyright(C) 2004                                                \/)\/    *
 * Visual Computing Lab                                            /\/|      *
 * ISTI - Italian National Research Council                           |      *
 *                                                                    \      *
 * All rights reserved.                                                      *
 *                                                                           *
 * This program is free software; you can redistribute it and/or modify      *   
 * it under the terms of the GNU General Public License as published by      *
 * the Free Software Foundation; either version 2 of the License, or         *
 * (at your option) any later version.                                       *
 *                                                                           *
 * This program is distributed in the hope that it will be useful,           *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 * GNU General Public License (http://www.gnu.org/licenses/gpl.txt)          *
 * for more details.                                                         *
 *                                                                           *
 ****************************************************************************/

#include "vcgmesh.h"
#include <wrap/io_trimesh/import.h>

// algorithms to update the bounding box and the normals of the mesh
#include <vcg/complex/trimesh/update/bounding.h>
#include <vcg/complex/trimesh/update/normal.h>

#include <QApplication>
#include <QDialog>
#include "glwidget.h"

int main(int argc, char *argv[])
{
    VCGMesh mesh;

    if (argc < 2)
    {
        std::cout << "Usage: trimesh <filename> " << std::endl;
        std::cout << "       <filename>  Mesh in PLY format" << std::endl << std::endl;
    }

    // Loading of the mesh from disk
    if(vcg::tri::io::Importer<VCGMesh>::Open(mesh,argv[1])!=0)
    {
      std::cout << "Error reading file %s\n" << argv[1] << std::endl << std::endl;
      exit(0);
    }

    // Initialize the mesh
    vcg::tri::UpdateBounding<VCGMesh>::Box(mesh);                        // update the bounding box
    vcg::tri::UpdateNormals<VCGMesh>::PerVertexNormalizedPerFace(mesh);  // update Normals
    vcg::tri::UpdateNormals<VCGMesh>::NormalizeFace(mesh);

    QApplication app(argc, argv);

    QDialog dlg;
    GLWidget *glwidget = new GLWidget(&mesh, &dlg);
    glwidget->setFixedSize(600,600);

    dlg.exec();

    return 0;
}
